@@ -5,17 +5,17 @@ import java.io._
 object Helper {
 
   def writeString(out: DataOutputStream, string: String): Unit = {
-    val b = string.getBytes
-    val l = b.length
-    out.writeInt(l)
-    out.write(b,0,l)
+    val bytes = string.getBytes("UTF-8")
+    val length = bytes.length
+    out.writeInt(length)
+    out.write(bytes,0,length)
   }
 
   def readString(in: DataInputStream): String = {
     val length = in.readInt()
     val bytes = new Array[Byte](length)
     in.readFully(bytes)
-    new String(bytes)
+    new String(bytes,"UTF-8")
   }
 
   def isMatrix[T](x: Array[Array[T]]): Boolean = {
