@@ -31,6 +31,9 @@ object Protocol {
   val DEF           = 110
   val INVOKE        = 111
   val SCALAP        = 112
+  val EVALNAKED     = 113
+  val SHUTDOWN      = 114
+  val SERIALIZE     = 115
 
   // Result
   val OK = 1000
@@ -38,7 +41,7 @@ object Protocol {
   val UNDEFINED_IDENTIFIER = 1002
 
   // Misc.
-  val CURRENT_SUPPORTED_SCALA_VERSION = "2.11.6"
+  val CURRENT_SUPPORTED_SCALA_VERSION = "2.11.8"
 
 }
 
