@@ -6,55 +6,185 @@ import scala.language.dynamics
 
 import Protocol._
 
-class RClient private (private val in: DataInputStream, private val out: DataOutputStream, private val debugger: Debugger) extends Dynamic {
+/** An interface to an R interpreter.
+*
+* An object `R` is the instance of this class available in a Scala interpreter created by calling the function
+* `scalaInterpreter` from the package [[http://cran.r-project.org/package=rscala rscala]].  It is through this instance `R` that
+* callbacks to the original [[http://www.r-project.org R]] interpreter are possible.
+* 
+* In a JVM-based application, an instance of this class is created using its companion object.  See below.  The paths of the
+* rscala's JARs, for both Scala 2.10 and 2.11, are available from [[http://www.r-project.org R]] using `rscala::rscalaJar()`.
+* To get just the JAR for Scala 2.11, for example, use `rscala::rscalaJar("2.11")`.
+* 
+* {{{
+* val R = org.ddahl.rscala.RClient()
 
-  def debug = debugger.debug
+* val a = R.evalD0("rnorm(8)")
+* val b = R.evalD1("rnorm(8)")
+* val c = R.evalD2("matrix(rnorm(8),nrow=4)")
 
+* R eval """
+*   v <- rbinom(8,size=10,prob=0.4)
+*   m <- matrix(v,nrow=4)
+* """
+* val v1 = R.get("v")
+* val v2 = R.get("v")._1.asInstanceOf[Array[Int]]   // This works, but is not very convenient
+* val v3 = R.v._1.asInstanceOf[Array[Int]]          // Slightly better
+* val v4 = R.getI0("v")   // Get the first element of R's "v" as a Int
+* val v5 = R.getI1("v")   // Get R's "v" as an Array[Int]
+* val v6 = R.getI2("m")   // Get R's "m" as an Array[Array[Int]]
+* }}}
+*
+* @author David B. Dahl
+*/
+class RClient private (private val scalaServer: ScalaServer, private val in: DataInputStream, private val out: DataOutputStream, private val debugger: Debugger, private val serializeOutputState: State) extends Dynamic {
+
+  /** __For rscala developers only__: Returns `TRUE` if debugging output is enabled. */
+  def debug = debugger.value
+
+  /** __For rscala developers only__: Sets whether debugging output should be displayed. */
   def debug_=(v: Boolean) = {
     if ( v != debug ) {
       if ( debug ) debugger.msg("Sending DEBUG request.")
       out.writeInt(DEBUG)
       out.writeInt(if ( v ) 1 else 0)
       out.flush()
-      debugger.debug = v
+      debugger.value = v
     }
   }
 
+  def serializeOutput = serializeOutputState.value
+
+  def serializeOutput_=(v: Boolean) = {
+    if ( scalaServer == null ) {
+      serializeOutputState.value = v
+    } else throw new IllegalStateException(s"Change this value in R via intpSettings(interpreter,serialize=${if (v) "TRUE" else "FALSE"})")
+  }
+
+  /** Closes the interface to the R interpreter.
+  * 
+  * Subsequent calls to the other methods will fail.
+  */
   def exit() = {
-    if ( debug ) debugger.msg("Sending EXIT request.")
-    out.writeInt(EXIT)
+    out.writeInt(SHUTDOWN)
     out.flush()
   }
 
+  /** Evaluates `snippet` in the R interpreter.
+  *
+  * Returns `null` if `evalOnly`.  If `!evalOnly`, the last result of the R expression is converted if possible.
+  * Conversion to integers, doubles, Booleans, and strings are supported, as are vectors (i.e. arrays) and matrices
+  * (i.e. retangular arrays of arrays) of these types.  The static type of the result, however, is `Any` so using the
+  * method `evalXY` (where `X` is `I`, `D`, `B`, or `S` and `Y` is `0`, `1`, or `2`) may be more convenient (e.g.
+  * [[evalD0]]).
+  */
   def eval(snippet: String, evalOnly: Boolean = true): Any = {
     if ( debug ) debugger.msg("Sending EVAL request.")
-    out.writeInt(EVAL)
+    out.writeInt(if(serializeOutput) EVAL else EVALNAKED)
     Helper.writeString(out,snippet)
     out.flush()
+    if ( scalaServer != null ) {
+      if ( debug ) debugger.msg("Spinning up Scala server.")
+      scalaServer.run()
+      if ( debug ) debugger.msg("Spinning down Scala server.")
+    }
     val status = in.readInt()
+    if ( debug ) debugger.msg("Status is: "+status)
     val output = Helper.readString(in)
-    if ( output != "" ) println(output)
+    if ( output != "" ) {
+      println(output)
+    } else if ( debug ) debugger.msg("No output.")
     if ( status != OK ) throw new RuntimeException("Error in R evaluation.")
     if ( evalOnly ) null else get(".rscala.last.value")._1
   }
 
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getI0]].  */
   def evalI0(snippet: String) = { eval(snippet,true); getI0(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getD0]].  */
   def evalD0(snippet: String) = { eval(snippet,true); getD0(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getB0]].  */
   def evalB0(snippet: String) = { eval(snippet,true); getB0(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getS0]].  */
   def evalS0(snippet: String) = { eval(snippet,true); getS0(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getI1]].  */
   def evalI1(snippet: String) = { eval(snippet,true); getI1(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getD1]].  */
   def evalD1(snippet: String) = { eval(snippet,true); getD1(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getB1]].  */
   def evalB1(snippet: String) = { eval(snippet,true); getB1(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getS1]].  */
   def evalS1(snippet: String) = { eval(snippet,true); getS1(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getI2]].  */
   def evalI2(snippet: String) = { eval(snippet,true); getI2(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getD2]].  */
   def evalD2(snippet: String) = { eval(snippet,true); getD2(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getB2]].  */
   def evalB2(snippet: String) = { eval(snippet,true); getB2(".rscala.last.value") }
+
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getS2]].  */
   def evalS2(snippet: String) = { eval(snippet,true); getS2(".rscala.last.value") }
 
+  /** Calls '''`eval(snippet,true)`''' and returns the result using [[getR]].  */
+  def evalR( snippet: String) = { eval(snippet,true); getR( ".rscala.last.value") }
+
+  /** A short-hand way to call [[get]].
+  *
+  * {{{
+  * R eval """
+  *   a <- numeric(10)
+  *   for ( i in 2:length(a) ) {
+  *     a[i] <- 0.5*a[i-1] + rnorm(1)
+  *   }
+  * """
+  * R.a
+  * }}}
+  */
   def selectDynamic(identifier: String): (Any,String) = get(identifier)
 
+  /** A short-hand way to call '''`set(identifier,value)`'''
+  *
+  * {{{
+  * R.b = Array.fill(10) { scala.math.random }
+  * }}}
+  */
   def updateDynamic(identifier: String)(value : Any): Unit = set(identifier,value)
+  
+  /** Equivalent to calling '''`set(identifier, value, "", true)`'''. */
+  def set(identifier: String, value: Any): Unit = set(identifier,value,"",true)
 
+  /** Assigns `value` to a variable `identifier` in the R interpreter.
+  * 
+  * Integers, doubles, Booleans, and strings are supported, as are vectors (i.e. arrays) and matrices
+  * (i.e. retangular arrays of arrays) of these types.
+  * 
+  * If `index != ""`, assigned into elements of `identifier` are performed by using either single brackets
+  * (`singleBrackets=true`) or double brackets (`singleBrackets=false`).
+  *
+  * {{{
+  * R.a = Array(5,6,4)
+  *
+  * R.eval("b <- matrix(NA,nrow=3,ncol=2)")
+  * for ( i <- 0 until 3 ) {
+  *   R.set("b",Array(2*i,2*i+1),s"\${i+1},")
+  * }
+  * R.b
+  *
+  * R.eval("myList <- list()")
+  * R.set("myList",Array("David","Grace","Susan"),"'names'",false)
+  * R.set("myList",Array(5,4,5),"'counts'",false)
+  * R.eval("print(myList)")
+  * }}}
+  */
   def set(identifier: String, value: Any, index: String = "", singleBrackets: Boolean = true): Unit = {
     if ( debug ) debugger.msg("Setting: "+identifier)
     val v = value
@@ -206,11 +336,29 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     }
   }
 
-  def get(identifier: String): (Any,String) = {
+  /** Returns the value of `identifier` in the R interpreter.  The static type of the result is `(Any,String)`, where
+  * the first element is the value and the second is the runtime type.
+  *
+  * If `asReference=false`, conversion to integers, doubles, Booleans, and strings are supported, as are vectors (i.e.
+  * arrays) and matrices (i.e. retangular arrays of arrays) of these types.    Using the method `getXY` (where `X` is
+  * `I`, `D`, `B`, or `S` and `Y` is `0`, `1`, or `2`) may be more convenient (e.g.  [[getD0]]).
+  *
+  * If `asReference=true`, the value is merely wrapped using [[RObject]] and objects of any type are supported.  Using
+  * the method [[getR]] may be more convenient.
+  */
+  def get(identifier: String, asReference: Boolean = false): (Any,String) = {
     if ( debug ) debugger.msg("Getting: "+identifier)
-    out.writeInt(GET)
+    out.writeInt(if ( asReference ) GET_REFERENCE else GET)
     Helper.writeString(out,identifier)
     out.flush()
+    if ( asReference ) {
+      val r = in.readInt() match {
+        case REFERENCE => (RObject(Helper.readString(in)),"RObject")
+        case UNDEFINED_IDENTIFIER =>
+          throw new RuntimeException("Undefined identifier")
+      }
+      return r
+    }
     in.readInt match {
       case NULLTYPE =>
         if ( debug ) debugger.msg("Getting null.")
@@ -253,6 +401,11 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     }
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to an `Int`.
+  *
+  * Integers, doubles, Booleans, and strings are supported.  Vectors (i.e. arrays) of these types are also supported by
+  * converting the first element.  Matrices (i.e. rectangular arrays of arrays) are not supported.
+  */
   def getI0(identifier: String): Int = get(identifier) match {
     case (a,"Int") => a.asInstanceOf[Int]
     case (a,"Double") => a.asInstanceOf[Double].toInt
@@ -265,6 +418,11 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Int")
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to a `Double`.
+  *
+  * Integers, doubles, Booleans, and strings are supported.  Vectors (i.e. arrays) of these types are also supported by
+  * converting the first element.  Matrices (i.e. rectangular arrays of arrays) are not supported.
+  */
   def getD0(identifier: String): Double = get(identifier) match {
     case (a,"Int") => a.asInstanceOf[Int].toDouble
     case (a,"Double") => a.asInstanceOf[Double]
@@ -277,6 +435,11 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Double")
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to a `Boolean`.
+  *
+  * Integers, doubles, Booleans, and strings are supported.  Vectors (i.e. arrays) of these types are also supported by
+  * converting the first element.  Matrices (i.e. rectangular arrays of arrays) are not supported.
+  */
   def getB0(identifier: String): Boolean = get(identifier) match {
     case (a,"Int") => a.asInstanceOf[Int] != 0
     case (a,"Double") => a.asInstanceOf[Double] != 0.0
@@ -289,6 +452,11 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Boolean")
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to a `string`.
+  *
+  * Integers, doubles, Booleans, and strings are supported.  Vectors (i.e. arrays) of these types are also supported by
+  * converting the first element.  Matrices (i.e. rectangular arrays of arrays) are not supported.
+  */
   def getS0(identifier: String): String = get(identifier) match {
     case (a,"Int") => a.asInstanceOf[Int].toString
     case (a,"Double") => a.asInstanceOf[Double].toString
@@ -301,6 +469,11 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to String")
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to an `Array[Int]`.
+  *
+  * Integers, doubles, Booleans, and strings are supported.  Vectors (i.e. arrays) of these types are also supported by
+  * converting the first element.  Matrices (i.e. rectangular arrays of arrays) are not supported.
+  */
   def getI1(identifier: String): Array[Int] = get(identifier) match {
     case (a,"Int") => Array(a.asInstanceOf[Int])
     case (a,"Double") => Array(a.asInstanceOf[Double].toInt)
@@ -313,6 +486,11 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Array[Int]")
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to an `Array[Double]`.
+  *
+  * Integers, doubles, Booleans, and strings are supported.  Vectors (i.e. arrays) of these types are also supported by
+  * converting the first element.  Matrices (i.e. rectangular arrays of arrays) are not supported.
+  */
   def getD1(identifier: String): Array[Double] = get(identifier) match {
     case (a,"Int") => Array(a.asInstanceOf[Int].toDouble)
     case (a,"Double") => Array(a.asInstanceOf[Double])
@@ -325,6 +503,11 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Array[Double]")
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to an `Array[Boolean]`.
+  *
+  * Integers, doubles, Booleans, and strings are supported.  Vectors (i.e. arrays) of these types are also supported by
+  * converting the first element.  Matrices (i.e. rectangular arrays of arrays) are not supported.
+  */
   def getB1(identifier: String): Array[Boolean] = get(identifier) match {
     case (a,"Int") => Array(a.asInstanceOf[Int] != 0)
     case (a,"Double") => Array(a.asInstanceOf[Double] != 0.0)
@@ -337,6 +520,11 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Array[Boolean]")
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to an `Array[string]`.
+  *
+  * Integers, doubles, Booleans, and strings are supported.  Vectors (i.e. arrays) of these types are also supported by
+  * converting the first element.  Matrices (i.e. rectangular arrays of arrays) are not supported.
+  */
   def getS1(identifier: String): Array[String] = get(identifier) match {
     case (a,"Int") => Array(a.asInstanceOf[Int].toString)
     case (a,"Double") => Array(a.asInstanceOf[Double].toString)
@@ -349,6 +537,12 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Array[String]")
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to an `Array[Array[Int]]`.
+  *
+  * Matrices (i.e. rectangular arrays of arrays) of integers, doubles, Booleans, and strings are supported.  Integers, doubles,
+  * Booleans, and strings themselves are not supported.  Vectors (i.e. arrays) of these
+  * types are also not supported.
+  */
   def getI2(identifier: String): Array[Array[Int]] = get(identifier) match {
     case (a,"Array[Array[Int]]") => a.asInstanceOf[Array[Array[Int]]]
     case (a,"Array[Array[Double]]") => a.asInstanceOf[Array[Array[Double]]].map(_.map(_.toInt))
@@ -357,7 +551,12 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Array[Array[Int]]")
   }
 
-
+  /** Calls '''`get(identifier,false)`''' and converts the result to an `Array[Array[Double]]`.
+  *
+  * Matrices (i.e. rectangular arrays of arrays) of integers, doubles, Booleans, and strings are supported.  Integers, doubles,
+  * Booleans, and strings themselves are not supported.  Vectors (i.e. arrays) of these
+  * types are also not supported.
+  */
   def getD2(identifier: String): Array[Array[Double]] = get(identifier) match {
     case (a,"Array[Array[Int]]") => a.asInstanceOf[Array[Array[Int]]].map(_.map(_.toDouble))
     case (a,"Array[Array[Double]]") => a.asInstanceOf[Array[Array[Double]]]
@@ -366,6 +565,12 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Array[Array[Double]]")
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to an `Array[Array[Boolean]]`.
+  *
+  * Matrices (i.e. rectangular arrays of arrays) of integers, doubles, Booleans, and strings are supported.  Integers, doubles,
+  * Booleans, and strings themselves are not supported.  Vectors (i.e. arrays) of these
+  * types are also not supported.
+  */
   def getB2(identifier: String): Array[Array[Boolean]] = get(identifier) match {
     case (a,"Array[Array[Int]]") => a.asInstanceOf[Array[Array[Int]]].map(_.map(_ != 0))
     case (a,"Array[Array[Double]]") => a.asInstanceOf[Array[Array[Double]]].map(_.map(_ != 0.0))
@@ -374,6 +579,12 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Array[Array[Boolean]]")
   }
 
+  /** Calls '''`get(identifier,false)`''' and converts the result to an `Array[Array[string]]`.
+  *
+  * Matrices (i.e. rectangular arrays of arrays) of integers, doubles, Booleans, and strings are supported.  Integers, doubles,
+  * Booleans, and strings themselves are not supported.  Vectors (i.e. arrays) of these
+  * types are also not supported.
+  */
   def getS2(identifier: String): Array[Array[String]] = get(identifier) match {
     case (a,"Array[Array[Int]]") => a.asInstanceOf[Array[Array[Int]]].map(_.map(_.toString))
     case (a,"Array[Array[Double]]") => a.asInstanceOf[Array[Array[Double]]].map(_.map(_.toString))
@@ -382,8 +593,39 @@ class RClient private (private val in: DataInputStream, private val out: DataOut
     case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to Array[Array[String]]")
   }
 
+  /** Calls '''`get(identifier,true)`''' and converts the result to an `[[RObject]]`.
+  *
+  * The value is merely wrapped using [[RObject]] and objects of any type are supported.
+  */
+  def getR(identifier: String): RObject = get(identifier,true) match {
+    case (a,"RObject") => a.asInstanceOf[RObject]
+    case (_,tp) => throw new RuntimeException(s"Unable to cast ${tp} to RObject")
+  }
+
+  /**
+  * Reclaims memory associated with __'''all'''__ R references, including any instances of [[RObject]] that are still in
+  * memory.
+  */
+  def gc(): Unit = {
+    if ( debug ) debugger.msg("Sending GC request.")
+    out.writeInt(GC)
+    out.flush()
+  }
+
 }
 
+/** The companion object to the [[RClient]] class used to create an instance of the [[RClient]] class in a JVM-based
+ * application.
+*
+* An object `R` is an [[RClient]] instance available in a Scala interpreter created by calling the function
+* `scalaInterpreter` from the package [[http://cran.r-project.org/package=rscala rscala]].  It is through this instance
+* `R` that callbacks to the original [[http://www.r-project.org R]] interpreter are possible.
+* 
+* The paths of the rscala's JARs, for both Scala 2.10 and 2.11, are available from [[http://www.r-project.org R]] using
+* `rscala::rscalaJar()`.  To get just the JAR for Scala 2.11, for example, use `rscala::rscalaJar("2.11")`.
+* 
+* {{{ val R = org.ddahl.rscala.RClient() }}}
+*/
 object RClient {
 
   import scala.sys.process._
@@ -397,17 +639,18 @@ object RClient {
   }
 
   private val defaultArguments = OS match {
-    case "windows" =>    Array[String]("--vanilla","--silent","--slave","--ess") 
-    case "linux" =>      Array[String]("--vanilla","--silent","--slave","--interactive")
-    case "unix" =>       Array[String]("--vanilla","--silent","--slave","--interactive")
-    case "macintosh" =>  Array[String]("--vanilla","--silent","--slave","--interactive")
+    case "windows" =>  Array[String]("--vanilla","--silent","--slave") 
+    case _ =>          Array[String]("--vanilla","--silent","--slave")
+  }
+
+  private val interactiveArguments = OS match {
+    case "windows" =>  Array[String]("--ess") 
+    case _ =>          Array[String]("--interactive")
   }
 
   private lazy val defaultRCmd = OS match {
-    case "windows" =>   findROnWindows
-    case "linux" =>     """R"""
-    case "unix" =>      """R"""
-    case "macintosh" => """R"""
+    case "windows" =>  findROnWindows
+    case _ =>          """R"""
   }
 
   private def findROnWindows: String = {
@@ -432,43 +675,65 @@ object RClient {
     val in = new BufferedReader(new InputStreamReader(input))
     var line = in.readLine()
     while ( line != null ) {
-      if ( debugger.debug ) println(label+line)
+      if ( debugger.value ) println(label+line)
       line = in.readLine()
     }
     in.close()
   }
 
+  /**
+  * Returns an instance of the [[RClient]] class by running the `R` executable in the path.
+  */
   def apply(): RClient = apply(defaultRCmd)
 
-  def apply(rCmd: String, debug: Boolean = false): RClient = {
+  /** Returns an instance of the [[RClient]] class using the path specified by `rCmd` and specifying whether debugging
+  * output should be display and the `timeout` to establish a connection with the R interpreter.
+  */
+  def apply(rCmd: String, debug: Boolean = false, serializeOutput: Boolean = false, timeout: Int = 60): RClient = {
     var cmd: PrintWriter = null
-    val command = rCmd +: defaultArguments
+    val command = rCmd +: ( defaultArguments ++ interactiveArguments )
     val processCmd = Process(command)
-    val debugger = new Debugger(debug)
+    val serializeOutputState = new State(serializeOutput)
+    val debugger = new Debugger(System.out,"Scala",false,debug)
     val processIO = new ProcessIO(
       o => { cmd = new PrintWriter(o) },
-      reader(debugger,"STDOUT DEBUG: "),
-      reader(debugger,"STDERR DEBUG: "),
+      reader(debugger,""),
+      reader(debugger,""),
       true
     )
-    val portsFile = File.createTempFile("rscala-","")
     val processInstance = processCmd.run(processIO)
+    val codeInR = Seq("common.R","globals.R","protocol.R","rServer.R","scalaInterpreter.R","zzz.R").map(resource => {
+      scala.io.Source.fromInputStream(getClass.getResourceAsStream("/R/"+resource)).getLines.mkString("\n")
+    }).mkString("\n\n")
+    val allCodeInR = s"""
+      rscala <- local({
+        ${codeInR}
+        environment()
+      })"""
+    val sourceFile = File.createTempFile("rscala-","")
+    val sourceFileNameForR = sourceFile.getAbsolutePath.replace(File.separator,"/")
+    val writer = new FileWriter(sourceFile)
+    writer.write(allCodeInR)
+    writer.flush()
+    writer.close()
+    val portsFile = File.createTempFile("rscala-","")
+    val snippet = s"""
+      source("${sourceFileNameForR}")
+      file.remove("${sourceFileNameForR}")
+      rscala[['rServe']](rscala[['newSockets']]('${portsFile.getAbsolutePath.replace(File.separator,"/")}',debug=${if ( debug ) "TRUE" else "FALSE"},serialize=${if ( serializeOutput ) "TRUE" else "FALSE"},timeout=${timeout}),with.callbacks=FALSE)
+      q(save='no')
+    """.stripMargin
     while ( cmd == null ) Thread.sleep(100)
-    cmd.println("library(rscala)")
-    cmd.print("rscala:::rServe(rscala:::newSockets('")
-    cmd.print(portsFile.getAbsolutePath.replaceAll(File.separator,"/"))
-    cmd.print("',debug=")
-    cmd.print( if ( debug ) "TRUE" else "FALSE" )
-    cmd.println("))")
-    cmd.println(raw"q(save='no')")
+    cmd.println(snippet)
     cmd.flush()
     val sockets = new ScalaSockets(portsFile.getAbsolutePath,debugger)
     sockets.out.writeInt(OK)
     sockets.out.flush()
-    apply(sockets.in,sockets.out,debugger)
+    apply(null,sockets.in,sockets.out,debugger,serializeOutputState)
   }
 
-  def apply(in: DataInputStream, out: DataOutputStream, debugger: Debugger): RClient = new RClient(in,out,debugger)
+  /** __For rscala developers only__: Returns an instance of the [[RClient]] class.  */
+  def apply(scalaServer: ScalaServer, in: DataInputStream, out: DataOutputStream, debugger: Debugger, serializeOutputState: State): RClient = new RClient(scalaServer,in,out,debugger,serializeOutputState)
 
 }
 
