@@ -2,29 +2,29 @@ package org.ddahl.rscala
 
 import java.net._
 import java.io._
-import scala.Console.{baosOut, baosErr, psOut, psErr, withOut, withErr}
 import scala.annotation.tailrec
 
 import Protocol._
 
-class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debugger: Debugger) {
+class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debugger: Debugger, serializeOutputState: State) {
 
   private val sockets = new ScalaSockets(portsFilename,debugger)
   import sockets.{in, out, socketIn, socketOut}
-  private val R = org.ddahl.rscala.RClient(in,out,debugger)
+  private val R = org.ddahl.rscala.RClient(this,in,out,debugger,serializeOutputState)
 
-  if ( repl == null ) out.writeInt(ERROR)
-  else {
+  if ( repl == null ) {
+    out.writeInt(ERROR)
+    out.flush
+    sys.error("REPL is null.")
+  } else {
     out.writeInt(OK)
+    out.flush
     repl.bind("R","org.ddahl.rscala.RClient",R)
     if ( repl.interpreter.isInstanceOf[scala.tools.nsc.interpreter.IMain] ) {
       repl.bind("$intp","scala.tools.nsc.interpreter.IMain",repl.interpreter)
+      repl.eval("import org.ddahl.rscala.RObject")
     }
   }
-  out.flush
-
-  baosOut.reset
-  baosErr.reset
 
   private var functionResult: (Any, String) = null
   private var wrapResult: Array[Array[String]] = null
@@ -70,27 +70,22 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
     val (f,m,nArgs,resultType) = functionMap(functionName)
     val functionParameters = functionParametersStack.takeRight(nArgs)
     functionParametersStack.trimEnd(nArgs)
-    if ( debugger.debug ) {
+    if ( debugger.value ) {
       debugger.msg("Function is: "+f)
       debugger.msg("There are "+functionParameters.length+" parameters.")
       debugger.msg("<<"+functionParameters.mkString(">>\n<<")+">>")
     }
-    functionResult = (m.invoke(f, functionParameters.map(_.asInstanceOf[AnyRef]): _*),resultType)
+    functionResult = (m.invoke(f, functionParameters.map(_.asInstanceOf[AnyRef]): _*), resultType)
   }
 
   private def readString() = Helper.readString(in)
-  private def writeString(string: String): Unit = Helper.writeString(out,string)
-
-  private def writeAll(buffer: StringBuffer): Unit = {
-    buffer.append(baosOut.toString)
-    buffer.append(baosErr.toString)
-    baosOut.reset
-    baosErr.reset
-    writeString(buffer.toString)
+  private def writeString(string: String): Unit = {
+    if ( debugger.value ) debugger.msg("Writing string: <"+string+">")
+    Helper.writeString(out,string)
   }
 
   private def setAVM(identifier: String, t: String, v: Any): Unit = {
-    if ( debugger.debug ) debugger.msg("Value is "+v)
+    if ( debugger.value ) debugger.msg("Value is "+v)
     if ( identifier == "." ) functionParametersStack.append(v)
     else repl.bind(identifier,t,v)
   }
@@ -98,9 +93,9 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
   private val sep = sys.props("line.separator")
 
   private def doGC(): Unit = {
-    if ( debugger.debug ) debugger.msg("Garbage collection")
+    if ( debugger.value ) debugger.msg("Garbage collection")
     val length = in.readInt()
-    if ( debugger.debug ) debugger.msg("... of length: "+length)
+    if ( debugger.value ) debugger.msg("... of length: "+length)
     for ( i <- 0 until length ) {
       cacheMap(in.readInt()) = null
     }
@@ -108,33 +103,26 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
 
   private def doEval(): Unit = {
     val snippet = readString()
-    val buffer = new StringBuffer()
     try {
-      withOut(psOut) {
-        withErr(psErr) {
-          repl.interpret(snippet)
-        }
-      }
+      repl.eval(snippet)
       R.exit()
-      if ( debugger.debug ) debugger.msg("Eval is okay")
+      if ( debugger.value ) debugger.msg("Eval is okay")
       out.writeInt(OK)
     } catch {
       case e: Throwable =>
+        if ( debugger.value ) debugger.msg("Caught throwable: "+e.toString)
+        repl.bind("$rscalamxception","Throwable",e) // So that repl.mostRecentVar is this throwable
         R.exit()
         out.writeInt(ERROR)
-        val errors = new StringWriter()
-        e.printStackTrace(new PrintWriter(errors))
-        buffer.append(errors.toString)
+        e.printStackTrace()
+        scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
     }
-    writeAll(buffer)
-    out.flush()
   }
 
   private def doDef(): Unit = {
     val args = readString().trim
     val body = readString()
-    if ( debugger.debug ) debugger.msg("Got arguments and body.")
-    val buffer = new StringBuffer()
+    if ( debugger.value ) debugger.msg("Got arguments and body.")
     try {
       val params = if ( args != "" ) {
         val y = args.split(":").map(z => {
@@ -151,46 +139,40 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
         }
         (r,s)
       } else (Array[String](),Array[String]())
-      if ( debugger.debug ) debugger.msg("Parsed arguments.")
+      if ( debugger.value ) debugger.msg("Parsed arguments.")
       out.writeInt(OK)
       try {
         val paramNames = params._1
         val paramTypes = params._2
         out.writeInt(OK)
         try {
-          withOut(psOut) {
-            withErr(psErr) {
-              repl.eval(s"($args) => { $body }")
-            }
-          }
+          repl.eval(s"($args) => { $body }")
           out.writeInt(OK)
           val functionName = repl.mostRecentVar
-          if ( debugger.debug ) debugger.msg("Name of function is: <"+functionName+">")
+          if ( debugger.value ) debugger.msg("Name of function is: <"+functionName+">")
           try {
             storeFunction(functionName,paramNames.length)
-            if ( debugger.debug ) debugger.msg("Stored function.")
+            if ( debugger.value ) debugger.msg("Stored function.")
             out.writeInt(OK)
             writeString(functionName)
-            if ( debugger.debug ) debugger.msg("Everything is okay in 'def'.")
+            if ( debugger.value ) debugger.msg("Everything is okay in 'def'.")
             out.writeInt(paramNames.length)
-            if ( debugger.debug ) debugger.msg("There are "+paramNames.length+" parameters.")
+            if ( debugger.value ) debugger.msg("There are "+paramNames.length+" parameters.")
             paramNames.foreach(writeString)
             paramTypes.foreach(writeString)
             writeString(functionMap(functionName)._4)
-            if ( debugger.debug ) debugger.msg("Done.")
+            if ( debugger.value ) debugger.msg("Done.")
           } catch {
             case e: Throwable =>
               out.writeInt(ERROR)
-              val errors = new StringWriter()
-              e.printStackTrace(new PrintWriter(errors))
-              buffer.append(errors.toString)
+              e.printStackTrace()
+              scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
           }
         } catch {
           case e: Throwable =>
             out.writeInt(ERROR)
-            val errors = new StringWriter()
-            e.printStackTrace(new PrintWriter(errors))
-            buffer.append(errors.toString)
+            // e.printStackTrace()
+            scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
         }
       } catch {
         case e: Throwable =>
@@ -200,33 +182,23 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
       case e: Throwable =>
         out.writeInt(ERROR)
     }
-    writeAll(buffer)
-    out.flush()
   }
 
   private def doInvoke(): Unit = {
     val functionName = readString()
-    val buffer = new StringBuffer()
     try {
       val f = repl.valueOfTerm(functionName).get
-      withOut(psOut) {
-        withErr(psErr) {
-          callFunction(functionName)
-        }
-      }
+      callFunction(functionName)
       R.exit()
-      if ( debugger.debug ) debugger.msg("Invoke is okay")
+      if ( debugger.value ) debugger.msg("Invoke is okay")
       out.writeInt(OK)
     } catch {
       case e: Throwable =>
         R.exit()
         out.writeInt(ERROR)
-        val errors = new StringWriter()
-        e.printStackTrace(new PrintWriter(errors))
-        buffer.append(errors.toString)
+        // e.printStackTrace()
+        scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
     }
-    writeAll(buffer)
-    out.flush()
   }
 
   private val RegexpVar = """^\s+(?:final )?\s*(?:override )?\s*var\s+([^(:]+):\s+([^=;]+).*""".r
@@ -250,33 +222,24 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
   }
 
   private def doScalap(): Unit = {
-    if ( debugger.debug ) debugger.msg("Doing scalap...")
+    if ( debugger.value ) debugger.msg("Doing scalap...")
     val itemName = readString()
-    if ( debugger.debug ) debugger.msg("... on "+itemName)
+    if ( debugger.value ) debugger.msg("... on "+itemName)
     val classpath = sys.props("sun.boot.class.path") + sys.props("path.separator") + sys.props("rscala.classpath")
-    if ( debugger.debug ) debugger.msg("... with classpath "+classpath)
-    import java.io.{ByteArrayOutputStream, PrintStream}
-    val baos = new ByteArrayOutputStream()
-    val ps = new PrintStream(baos,true)
-    scala.Console.withOut(ps) {
-      scala.tools.scalap.Main.main(Array("-cp",classpath,itemName))
-    }
-    writeString(baos.toString)
-    out.flush()
-    baos.close()
-    ps.close()
+    if ( debugger.value ) debugger.msg("... with classpath "+classpath)
+    scala.tools.scalap.Main.main(Array("-cp",classpath,itemName))
+    out.writeInt(OK)
   }
 
   private def doSet(): Unit = {
     val identifier = readString()
-    val buffer = new StringBuffer()
     in.readInt() match {
       case NULLTYPE =>
-        if ( debugger.debug ) debugger.msg("Setting null.")
+        if ( debugger.value ) debugger.msg("Setting null.")
         if ( identifier == "." ) functionParametersStack.append(null)
         else repl.bind(identifier,"Any",null)
       case REFERENCE =>
-        if ( debugger.debug ) debugger.msg("Setting reference.")
+        if ( debugger.value ) debugger.msg("Setting reference.")
         val originalIdentifier = readString()
         try {
           if ( identifier == "." ) originalIdentifier match {
@@ -297,25 +260,20 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
                 repl.bind(r._1,originalType,originalValue)
               case _ =>
                 val vt = if ( r._2 == "" ) r._1 else r._1 + ": " + r._2
-                withOut(psOut) {
-                  withErr(psErr) {
-                    repl.eval(s"val ${vt} = ${originalIdentifier}")
-                  }
-                }
+                repl.eval(s"val ${vt} = ${originalIdentifier}")
             }
           }
           out.writeInt(OK)
         } catch {
           case e: Throwable =>
-            if ( debugger.debug ) debugger.msg("Caught exception: "+e)
+            if ( debugger.value ) debugger.msg("Caught exception: "+e)
             out.writeInt(ERROR)
-            val errors = new StringWriter()
-            e.printStackTrace(new PrintWriter(errors))
-            buffer.append(errors.toString)
+            e.printStackTrace()
+            scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
         }
       case ATOMIC =>
-        if ( debugger.debug ) debugger.msg("Setting atomic.")
-        val (v,t) = in.readInt() match {
+        if ( debugger.value ) debugger.msg("Setting atomic.")
+        val (v: Any, t: String) = in.readInt() match {
           case INTEGER => (in.readInt(),"Int")
           case DOUBLE => (in.readDouble(),"Double")
           case BOOLEAN => (( in.readInt() != 0 ),"Boolean")
@@ -324,10 +282,10 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
         }
         setAVM(identifier,t,v)
       case VECTOR =>
-        if ( debugger.debug ) debugger.msg("Setting vector...")
+        if ( debugger.value ) debugger.msg("Setting vector...")
         val length = in.readInt()
-        if ( debugger.debug ) debugger.msg("... of length: "+length)
-        val (v,t) = in.readInt() match {
+        if ( debugger.value ) debugger.msg("... of length: "+length)
+        val (v, t): (Any,String) = in.readInt() match {
           case INTEGER => (Array.fill(length) { in.readInt() },"Array[Int]")
           case DOUBLE => (Array.fill(length) { in.readDouble() },"Array[Double]")
           case BOOLEAN => (Array.fill(length) { ( in.readInt() != 0 ) },"Array[Boolean]")
@@ -336,11 +294,11 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
         }
         setAVM(identifier,t,v)
       case MATRIX =>
-        if ( debugger.debug ) debugger.msg("Setting matrix...")
+        if ( debugger.value ) debugger.msg("Setting matrix...")
         val nrow = in.readInt()
         val ncol = in.readInt()
-        if ( debugger.debug ) debugger.msg("... of dimensions: "+nrow+","+ncol)
-        val (v,t) = in.readInt() match {
+        if ( debugger.value ) debugger.msg("... of dimensions: "+nrow+","+ncol)
+        val (v, t): (Any,String) = in.readInt() match {
           case INTEGER => (Array.fill(nrow) { Array.fill(ncol) { in.readInt() } },"Array[Array[Int]]")
           case DOUBLE => (Array.fill(nrow) { Array.fill(ncol) { in.readDouble() } },"Array[Array[Double]]")
           case BOOLEAN => (Array.fill(nrow) { Array.fill(ncol) { ( in.readInt() != 0 ) } },"Array[Array[Boolean]]")
@@ -350,13 +308,11 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
         setAVM(identifier,t,v)
       case _ => throw new RuntimeException("Protocol error")
     }
-    writeAll(buffer)
-    out.flush()
   }
 
   private def doGet(): Unit = {
     val identifier = readString()
-    if ( debugger.debug ) debugger.msg("Trying to get value of: "+identifier)
+    if ( debugger.value ) debugger.msg("Trying to get value of: "+identifier)
     val optionWithType = try {
       identifier match {
         case "." => 
@@ -374,35 +330,32 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
       }
     } catch {
       case e: Throwable =>
-        if ( debugger.debug ) debugger.msg("Caught exception: "+e)
+        if ( debugger.value ) debugger.msg("Caught exception: "+e)
         out.writeInt(ERROR)
-        val errors = new StringWriter()
-        e.printStackTrace(new PrintWriter(errors))
-        val buffer = new StringBuffer()
-        buffer.append(errors.toString)
-        writeAll(buffer)
-        out.flush()
+        e.printStackTrace()
+        scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
         return
     }
-    if ( debugger.debug ) debugger.msg("Getting: "+identifier)
+    if ( debugger.value ) debugger.msg("Getting: "+identifier)
     if ( optionWithType._1.isEmpty ) {
-      if ( debugger.debug ) debugger.msg("... which does not exist.")
+      if ( debugger.value ) debugger.msg("... which does not exist.")
       out.writeInt(UNDEFINED_IDENTIFIER)
-      val buffer = new StringBuffer()
-      writeAll(buffer)
-      out.flush()
       return
     }
     val v = optionWithType._1.get
     if ( v == null || v.isInstanceOf[Unit] ) {
-      if ( debugger.debug ) debugger.msg("... which is null")
+      if ( debugger.value ) debugger.msg("... which is null")
       out.writeInt(NULLTYPE)
-      out.flush
+      return
+    }
+    if ( optionWithType._2 == "org.ddahl.rscala.RObject" ) {
+      out.writeInt(REFERENCE)
+      writeString(v.asInstanceOf[org.ddahl.rscala.RObject].name)
       return
     }
     val t = if ( optionWithType._2 == "Any" ) {
       val c = v.getClass
-      if ( debugger.debug ) debugger.msg("Trying to infer type of "+c)
+      if ( debugger.value ) debugger.msg("Trying to infer type of "+c)
       if ( c.isArray ) {
         c.getName match {
           case "[I" => "Array[Int]"
@@ -425,8 +378,8 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
         }
       }
     } else optionWithType._2
-    if ( debugger.debug ) debugger.msg("... whose type is: "+t)
-    if ( debugger.debug ) debugger.msg("... and whose value is: "+v)
+    if ( debugger.value ) debugger.msg("... whose type is: "+t)
+    if ( debugger.value ) debugger.msg("... and whose value is: "+v)
     t match {
       case "Array[Int]" =>
         val vv = v.asInstanceOf[Array[Int]]
@@ -529,15 +482,14 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
         out.writeInt(STRING)
         writeString(v.asInstanceOf[String])
       case _ =>
-        if ( debugger.debug ) debugger.msg("Bowing out: "+identifier)
+        if ( debugger.value ) debugger.msg("Bowing out: "+identifier)
         out.writeInt(UNSUPPORTED_STRUCTURE)
     }
-    out.flush()
   }
 
   private def doGetReference(): Unit = {
     val identifier = readString()
-    if ( debugger.debug ) debugger.msg("Trying to get reference for: "+identifier)
+    if ( debugger.value ) debugger.msg("Trying to get reference for: "+identifier)
     identifier match {
       case "?" =>
         cacheMap.append(functionResult)
@@ -554,7 +506,6 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
           out.writeInt(OK)
           writeString("null")
           writeString("Null")
-          out.flush()
           return
         }
         val id = if ( identifier == "." ) repl.mostRecentVar else identifier
@@ -562,14 +513,10 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
           repl.valueOfTerm(id)
         } catch {
           case e: Throwable =>
-            if ( debugger.debug ) debugger.msg("Caught exception: "+e)
+            if ( debugger.value ) debugger.msg("Caught exception: "+e)
             out.writeInt(ERROR)
-            val errors = new StringWriter()
-            e.printStackTrace(new PrintWriter(errors))
-            val buffer = new StringBuffer()
-            buffer.append(errors.toString)
-            writeAll(buffer)
-            out.flush()
+            e.printStackTrace()
+            scala.Console.err.println("Exception message: "+(if (e.getCause != null) e.getCause.getMessage else e.getMessage)+System.lineSeparator)
             return
         }
         if ( option.isEmpty ) out.writeInt(UNDEFINED_IDENTIFIER)
@@ -579,44 +526,35 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
           writeString(repl.typeOfTerm(id))
         }
     }
-    out.flush()
   }
 
-  @tailrec
-  final def run(): Unit = {
-    if ( repl == null ) {
-      if ( debugger.debug ) debugger.msg("Exiting")
-      in.close()
-      out.close()
-      socketIn.close()
-      socketOut.close()
-      return
-    }
-    if ( debugger.debug ) debugger.msg("Top of the loop waiting for a command.")
-    val request = try {
-      in.readInt()
-    } catch {
-      case _: Throwable =>
-        return
-    }
-    if ( debugger.debug ) debugger.msg("Received request: "+request)
+  private val originalOut = java.lang.System.out
+  private val originalErr = java.lang.System.err
+
+  private def heart(request: Int): Boolean = {
+    if ( debugger.value ) debugger.msg("Received request: "+request)
     request match {
-      case EXIT =>
-        if ( debugger.debug ) debugger.msg("Exiting")
+      case SHUTDOWN =>
+        if ( debugger.value ) debugger.msg("Shutting down")
         in.close()
         out.close()
         socketIn.close()
         socketOut.close()
-        return
+        return false
+      case EXIT =>
+        if ( debugger.value ) debugger.msg("Exiting")
+        return false
       case RESET =>
-        if ( debugger.debug ) debugger.msg("Resetting")
+        if ( debugger.value ) debugger.msg("Resetting")
         cacheMap.clear()
       case GC =>
         doGC()
       case DEBUG =>
-        val newDebug = ( in.readInt() != 0 )
-        if ( debugger.debug != newDebug ) debugger.msg("Debugging is now "+newDebug)
-        debugger.debug = newDebug
+        debugger.value = ( in.readInt() != 0 )
+        if ( debugger.value ) debugger.msg("Debugging is now "+debugger.value)
+      case SERIALIZE =>
+        serializeOutputState.value = ( in.readInt() != 0 )
+        if ( debugger.value ) debugger.msg("Serialize output is now "+serializeOutputState.value)
       case EVAL =>
         doEval()
       case SET =>
@@ -632,14 +570,43 @@ class ScalaServer private (repl: InterpreterAdapter, portsFilename: String, debu
       case SCALAP =>
         doScalap()
     }
-    run()
+    true
+  }
+
+  final def run(): Unit = {
+    while ( true ) {
+      if ( debugger.value ) debugger.msg("Scala server at top of the loop waiting for a command.")
+      val request = try {
+        in.readInt()
+      } catch {
+        case _: Throwable =>
+          return
+      }
+      if ( serializeOutputState.value ) {
+        val baosOut = new java.io.ByteArrayOutputStream()
+        val baosErr = new java.io.ByteArrayOutputStream()
+        val psOut = new java.io.PrintStream(baosOut,true)
+        val psErr = new java.io.PrintStream(baosErr,true)
+        System.setOut(psOut)
+        System.setErr(psErr)
+        scala.Console.withOut(psOut) {
+          scala.Console.withErr(psErr) {
+            if ( ! heart(request) ) return
+          }
+        }
+        writeString(baosOut.toString+baosErr.toString)
+      } else {
+        if ( ! heart(request) ) return
+      }
+      out.flush()
+    }
   }
 
 }
 
 object ScalaServer {
 
-  def apply(repl: InterpreterAdapter, portsFilename: String, debug: Boolean = false): ScalaServer = new ScalaServer(repl, portsFilename, new Debugger(debug))
+  def apply(repl: InterpreterAdapter, portsFilename: String, debug: Boolean = false, serializeOutput: Boolean = false): ScalaServer = new ScalaServer(repl, portsFilename, new Debugger(System.out,"Scala",false,debug), new State(serializeOutput))
 
 }
 
