@@ -22,6 +22,9 @@ GET_REFERENCE <- 109L
 DEF           <- 110L
 INVOKE        <- 111L
 SCALAP        <- 112L
+EVALNAKED     <- 113L
+SHUTDOWN      <- 114L
+SERIALIZE     <- 115L
 OK <- 1000L
 ERROR <- 1001L
 UNDEFINED_IDENTIFIER <- 1002L
